/*
 *
 * Copyright 2012-2013 Philip Quinn. All Rights Reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
//
//  WindowController.m
//  Scroll Gain Inspector
//

#import <ApplicationServices/ApplicationServices.h>
#import "WindowController.h"
#import "devices.hpp"
#import "experimental.hpp"

@interface WindowController ()
-(void)scroll:(double)amount;
@end

@implementation WindowController
@synthesize vendorId;
@synthesize productId;
@synthesize systemGain;

-(void)dealloc
{
    if (_eventTap)
        CFRelease(_eventTap);
    if (_device)
        delete _device;
}

-(void)scroll:(double)amount
{
    if (_scroller) {
        _scroller->recordScroll(amount);
    }
}

static CGEventRef scrollEvent(CGEventTapProxy proxy, CGEventType type, CGEventRef event, void *context) {
    double amount = CGEventGetDoubleValueField(event, kCGScrollWheelEventFixedPtDeltaAxis1);
    id obj = (__bridge id)context;
    
    [obj scroll:amount];
    return event;
}

-(IBAction)startScrolling:(id)sender {
    
    
    NSScanner *vScanner = [NSScanner scannerWithString:[self.vendorId stringValue]];
    NSScanner *pScanner = [NSScanner scannerWithString:[self.productId stringValue]];
    
    unsigned int vid = 0;
    unsigned int pid = 0;
    [vScanner scanHexInt:&vid];
    [pScanner scanHexInt:&pid];
    
    // Setup EchoMouse
    _device = new ScrollingDevice(vid, pid);
    if (!_device->open()) {
        NSAlert *a = [NSAlert alertWithMessageText:@"Cannot connect to EchoMouse"
                                     defaultButton:nil
                                   alternateButton:nil
                                       otherButton:nil
                         informativeTextWithFormat:@"Cannot open EchoMouse device."];
        [a beginSheetModalForWindow:self.window
                      modalDelegate:nil
                     didEndSelector:NULL
                        contextInfo:NULL];
        delete _device;
        _device = nullptr;
        return;
    } else {
        [sender setEnabled:NO];
    }
        
    // Create monitor for scroll events
    _eventTap = CGEventTapCreate(kCGHIDEventTap,
                                 kCGHeadInsertEventTap,
                                 kCGEventTapOptionListenOnly,
                                 CGEventMaskBit(kCGEventScrollWheel),
                                 scrollEvent, 
                                 (__bridge void*)self);
    if (!_eventTap) {
        NSAlert *a = [NSAlert alertWithMessageText:@"Error creating event tap"
                                     defaultButton:nil
                                   alternateButton:nil
                                       otherButton:nil
                         informativeTextWithFormat:@"This application must be run as an Administrator to monitor system scroll events."];
        [a beginSheetModalForWindow:self.window
                      modalDelegate:nil
                     didEndSelector:NULL
                        contextInfo:NULL];
        CFRelease(_eventTap);
        _eventTap = NULL;
        delete _device;
        _device = nullptr;
    }
    CFRunLoopSourceRef s = CFMachPortCreateRunLoopSource(NULL, _eventTap, 0);
    CFRunLoopAddSource(CFRunLoopGetMain(), s, kCFRunLoopDefaultMode);
    CFRelease(s);
        
    _scroller = new VelocityScroller(*(_device), -1, -200); 
    //_scroller = new DirectionScroller(*(_device));
    //_scroller = new ClutchScroller(*(_device));
    //_scroller = new MaintainVelocityScroller(*(_device), -400);
    //_scroller = new SlidingVelocityScroller(*(_device), -1);
    _scroller->run();
}

-(IBAction)selectPreset:(id)sender {
    NSPopUpButton *menu = (NSPopUpButton*)sender;
    NSInteger tag = [[menu selectedItem] tag];
    if (tag == -1 || tag == 0) {
        [self.vendorId setStringValue:@"0x4d8"];
        [self.productId setStringValue:@"0x3f"];
    } else if (tag == 1) {
        [self.vendorId setStringValue:@"0x45e"];
        [self.productId setStringValue:@"0x40"];
    } else if (tag == 2) {
        [self.vendorId setStringValue:@"0x46d"];
        [self.productId setStringValue:@"0xc01e"];
    } else if (tag == 3) {
        [self.vendorId setStringValue:@"0x46d"];
        [self.productId setStringValue:@"0xc025"];
    } else if (tag == 4) {
        [self.vendorId setStringValue:@"0x46d"];
        [self.productId setStringValue:@"0xc069"];
    } else if (tag == 5) {
        [self.vendorId setStringValue:@"0x4b3"];
        [self.productId setStringValue:@"0x3108"];
    }
}

@end
